/*********************************************************
*         SD21 Example program for PIC24FJ16GA002        *
*			   Moves all servos left and right           *
*********************************************************/

#include <p24fxxxx.h> 
#include "SD21.h"
#include "stdlib.h"
#include "stdio.h"

_CONFIG1(FWDTEN_OFF & JTAGEN_OFF & GCP_ON)
_CONFIG2(FNOSC_FRCPLL & OSCIOFNC_ON & IOL1WAY_ON & I2C1SEL_SEC) 

int main(void) 
{
unsigned char servoReg;
extern unsigned char s[21];
extern unsigned char buffer[36];

	setup_cpu();								// General chip setup
	StartTimer(0x186A);							// Pause to wait for everything to  power up
	WaitTimer();	
	setup_lcd();
	while(1)									// Loop here forever
	{
		read_i2c_buffer(0xc2, 64,2);				//read 2 bytes into buffer from sd21, reg 64 (software version) and reg 65 (battery voltage). Placing them in buffer[0] and buffer[1]
		set_cursor(1);								// Set the lcd03 cursor to position 1
		sprintf(s,"SD21 V%2d",buffer[0]);			// makes an ascii formatted string in s[]
		write_i2c_s_string(0xc6,0);					// sends ascii string s[] to lcd03
		set_cursor(21);
		sprintf(s,"Battery volts = %3d",buffer[1]);
		write_i2c_s_string(0xc6,0);					// print to the LCD03
		set_cursor(41);
		sprintf(s,"Servos to the left");			// Make asci strint "servos to the left"
		write_i2c_s_string(0xc6,0);					// Send it to the LCD03
		for(servoReg = 63; servoReg <= 83; servoReg++)
		{
			write_i2c(0xc2,servoReg,0);					// Change the position of all the servos	
		}

		StartTimer(0x7A12);								// Wait 20mS before changing the servo states, this time can be altered depending how fast you want the servos to change state	
		WaitTimer();

		set_cursor(41);
		sprintf(s,"Servos to the right");				// Make ascii string "to the right"
		write_i2c_s_string(0xc6,0);						// And display it to the LCD03
		for(servoReg = 63; servoReg <= 83; servoReg++)
		{
			write_i2c(0xc2,servoReg,255);				// Change the position of all the servos
		}

		StartTimer(0x7A12);								// Wait 20mS before changing servo states
		WaitTimer();									// Wait for timer to finish
	}
	return(0);
}

/**********************************
* Timer routines
**********************************/
void StartTimer(unsigned int time)
{
	TMR1 = 0;						// Reset timer to 0
	PR1 = time;						// Timer1 period set
	IFS0bits.T1IF = 0;				// Clears Timer 1 interupt flag
	IEC0bits.T1IE = 0;				// Disable timer 1 interupts 
	T1CONbits.TON = 1;				// Starts timer
}

void WaitTimer(void)
{
	while(!IFS0bits.T1IF);			// While the interupt flag is not set
	T1CONbits.TON = 0;				// Stop timer
}

/**************************************
* General Chip setup
**************************************/
void setup_cpu(void)
{
	CLKDIV = 0x0000;				// 8MHz to PLL, 32MHz out = 16MIPs
	T1CON = 0x30;					// 1:256 prescaler
	AD1PCFG = 0x9fff;				
}	



